@@ -1,10 +1,12 @@
+from .filehandlers import JsonFilehandler
+
 from .keyring import Key, Keyring, validator
 
 from .minivault import MiniVault
 
 from .utils import *
 
-from .vault import VarVault
+from .vault import VarVault, VarVaultInterface
 
 from .vaultflags import VaultFlags
 
@@ -14,10 +16,16 @@ from .vaultfactory import from_vault
 from .vaultstructs import *
 
 
+class FileTypes:
+    JSON: Type[JsonFilehandler] = JsonFilehandler
+
+
 def clear_logs():
     import os
     import glob
+    import tempfile
 
-    files = glob.glob("/tmp/varvault-logs/*.log")
+    path = os.path.join(tempfile.tempdir, "varvault-logs", "*.log")
+    files = glob.glob(path)
     for f in files:
         os.remove(f)
